  
#ifndef _atmosphere_cmpt_h_
#define _atmosphere_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/atmosphere/atmosphere_forward.h>

using namespace std;

namespace casac {
           


   class  atmosphere  {
       public :
static Quantity _altitude_getBasicAtmParms;
static Quantity _temperature_getBasicAtmParms;
static Quantity _pressure_getBasicAtmParms;
static Quantity _maxAltitude_getBasicAtmParms;

static double _humidity_getBasicAtmParms;
		static Quantity _dTem_dh_getBasicAtmParms;
static Quantity _dP_getBasicAtmParms;

static double _dPm_getBasicAtmParms;
		static Quantity _h0_getBasicAtmParms;

static string _atmType_getBasicAtmParms;
		static Quantity _thickness_getProfile;
static Quantity _temperature_getProfile;
static Quantity _watermassdensity_getProfile;
static Quantity _water_getProfile;
static Quantity _pressure_getProfile;
static Quantity _O3_getProfile;
static Quantity _CO_getProfile;
static Quantity _N2O_getProfile;
static std::vector<double> _dryOpacity_getDryOpacitySpec;
static Quantity _wetOpacity_getWetOpacitySpec;
static Quantity _tebbSky_getTebbSkySpec;
static Quantity _trjSky_getTrjSkySpec;


         ~atmosphere();
         atmosphere();

         bool close();

         bool done();

         string getAtmVersion();

         std::vector<std::string> listAtmosphereTypes();

         string initAtmProfile( const Quantity& altitude=Quantity(std::vector<double>(1, 5000.),"m"),  const Quantity& temperature=Quantity(std::vector<double>(1, 270.0),"K"),  const Quantity& pressure=Quantity(std::vector<double>(1, 560.0),"mbar"),  const Quantity& maxAltitude=Quantity(std::vector<double>(1, 48.0),"km"), 
			  double humidity=20.0,  const Quantity& dTem_dh=Quantity(std::vector<double>(1, -5.6),"K/km"),  const Quantity& dP=Quantity(std::vector<double>(1, 10.0),"mbar"), 
			  double dPm=1.2,  const Quantity& h0=Quantity(std::vector<double>(1, 2.0),"km"),  long  atmType=1,  const std::vector<double>& layerBoundaries=std::vector<double>() ,  const std::vector<double>& layerTemperature=std::vector<double>() );

         string updateAtmProfile( const Quantity& altitude=Quantity(std::vector<double>(1, 5000.),"m"),  const Quantity& temperature=Quantity(std::vector<double>(1, 270.0),"K"),  const Quantity& pressure=Quantity(std::vector<double>(1, 560.0),"mbar"), 
			  double humidity=20.0,  const Quantity& dTem_dh=Quantity(std::vector<double>(1, -5.6),"K/km"),  const Quantity& h0=Quantity(std::vector<double>(1, 2.0),"km"));

         string getBasicAtmParms( Quantity& _altitude=atmosphere::_altitude_getBasicAtmParms,  Quantity& _temperature=atmosphere::_temperature_getBasicAtmParms,  Quantity& _pressure=atmosphere::_pressure_getBasicAtmParms,  Quantity& _maxAltitude=atmosphere::_maxAltitude_getBasicAtmParms, 
			  double& _humidity=atmosphere::_humidity_getBasicAtmParms,  Quantity& _dTem_dh=atmosphere::_dTem_dh_getBasicAtmParms,  Quantity& _dP=atmosphere::_dP_getBasicAtmParms, 
			  double& _dPm=atmosphere::_dPm_getBasicAtmParms,  Quantity& _h0=atmosphere::_h0_getBasicAtmParms, string& _atmType=atmosphere::_atmType_getBasicAtmParms);

         long getNumLayers();

         Quantity getGroundWH2O();

         string getProfile( Quantity& _thickness=atmosphere::_thickness_getProfile,  Quantity& _temperature=atmosphere::_temperature_getProfile,  Quantity& _watermassdensity=atmosphere::_watermassdensity_getProfile,  Quantity& _water=atmosphere::_water_getProfile,  Quantity& _pressure=atmosphere::_pressure_getProfile,  Quantity& _O3=atmosphere::_O3_getProfile,  Quantity& _CO=atmosphere::_CO_getProfile,  Quantity& _N2O=atmosphere::_N2O_getProfile);

         long initSpectralWindow( long  nbands=1,  const Quantity& fCenter=Quantity(std::vector<double>(1, 90),"GHz"),  const Quantity& fWidth=Quantity(std::vector<double>(1, 0.64),"GHz"),  const Quantity& fRes=Quantity(std::vector<double>(1, 0.0),"GHz"));

         long addSpectralWindow( const Quantity& fCenter=Quantity(std::vector<double>(1, 350),"GHz"),  const Quantity& fWidth=Quantity(std::vector<double>(1, 0.008),"GHz"),  const Quantity& fRes=Quantity(std::vector<double>(1, 0.002),"GHz"));

         long getNumSpectralWindows();

         long getNumChan( long  spwid=0);

         long getRefChan( long  spwid=0);

         Quantity getRefFreq( long  spwid=0);

         Quantity getChanSep( long  spwid=0);

         Quantity getChanFreq( long  chanNum=0,  long  spwid=0);

         Quantity getSpectralWindow( long  spwid=0);

         double getChanNum( const Quantity& freq,  long  spwid=0);

         Quantity getBandwidth( long  spwid=0);

         Quantity getMinFreq( long  spwid=0);

         Quantity getMaxFreq( long  spwid=0);

         double getDryOpacity( long  nc=-1,  long  spwid=0);

         double getDryContOpacity( long  nc=-1,  long  spwid=0);

         double getO2LinesOpacity( long  nc=-1,  long  spwid=0);

         double getO3LinesOpacity( long  nc=-1,  long  spwid=0);

         double getCOLinesOpacity( long  nc=-1,  long  spwid=0);

         double getN2OLinesOpacity( long  nc=-1,  long  spwid=0);

         Quantity getWetOpacity( long  nc=-1,  long  spwid=0);

         double getH2OLinesOpacity( long  nc=-1,  long  spwid=0);

         double getH2OContOpacity( long  nc=-1,  long  spwid=0);

         long getDryOpacitySpec( long  spwid=0,  std::vector<double>& _dryOpacity=atmosphere::_dryOpacity_getDryOpacitySpec);

         long getWetOpacitySpec( long  spwid=0,  Quantity& _wetOpacity=atmosphere::_wetOpacity_getWetOpacitySpec);

         Quantity getDispersivePhaseDelay( long  nc=-1,  long  spwid=0);

         Quantity getDispersiveWetPhaseDelay( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersiveWetPhaseDelay( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersiveDryPhaseDelay( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersivePhaseDelay( long  nc=-1,  long  spwid=0);

         Quantity getDispersivePathLength( long  nc=-1,  long  spwid=0);

         Quantity getDispersiveWetPathLength( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersiveWetPathLength( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersiveDryPathLength( long  nc=-1,  long  spwid=0);

         Quantity getO2LinesPathLength( long  nc=-1,  long  spwid=0);

         Quantity getO3LinesPathLength( long  nc=-1,  long  spwid=0);

         Quantity getCOLinesPathLength( long  nc=-1,  long  spwid=0);

         Quantity getN2OLinesPathLength( long  nc=-1,  long  spwid=0);

         Quantity getNonDispersivePathLength( long  nc=-1,  long  spwid=0);

         Quantity getAbsH2OLines( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsH2OCont( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsO2Lines( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsDryCont( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsO3Lines( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsCOLines( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsN2OLines( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsTotalDry( long  nl,  long  nf=0,  long  spwid=0);

         Quantity getAbsTotalWet( long  nl,  long  nf=0,  long  spwid=0);

         bool setUserWH2O( const Quantity& wh2o=Quantity(std::vector<double>(1, 0.0),"mm"));

         Quantity getUserWH2O();

         bool setAirMass(
			  double airmass);

         double getAirMass();

         bool setSkyBackgroundTemperature( const Quantity& tbgr=Quantity(std::vector<double>(1, 2.73),"K"));

         Quantity getSkyBackgroundTemperature();

         Quantity getAverageTebbSky( long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));

         Quantity getTebbSky( long  nc=-1,  long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));

         long getTebbSkySpec( long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"),  Quantity& _tebbSky=atmosphere::_tebbSky_getTebbSkySpec);

         Quantity getAverageTrjSky( long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));

         Quantity getTrjSky( long  nc=-1,  long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"));

         long getTrjSkySpec( long  spwid=0,  const Quantity& wh2o=Quantity(std::vector<double>(1, -1),"mm"),  Quantity& _trjSky=atmosphere::_trjSky_getTrjSkySpec);


      private :
#include <tools/atmosphere/atmosphere_private.h>
   }; 

 
}
#endif
