  
#ifndef _componentlist_cmpt_h_
#define _componentlist_cmpt_h_

#include <vector>
#include <string>
#include <complex>
#include <stdcasa/record.h>
#include <tools/casaswig_types.h>
#include <casa/Exceptions/Error.h>
#include <tools/components/componentlist_forward.h>

using namespace std;

namespace casac {
           


   class  componentlist  {
       public :


         ~componentlist();
         componentlist();

         bool open(const  string& filename=string(""),  bool  nomodify=false,  bool  log=true);

         long asciitocomponentlist(const  string& filename, const  string& asciifile, const  string& refer=string("J2000"), const  string& format=string("ST"),  const record& direction=initialize_record("") ,  const record& spectrum=initialize_record("") ,  const record& flux=initialize_record("") ,  bool  log=true);

         bool concatenate( const variant & list= variant(),  const std::vector<long>& which=initialize_vector(1, (long)-1),  bool  log=true);

         bool fromrecord( const record& record=initialize_record("") );

         record* torecord();

         bool remove( const std::vector<long>& which=initialize_vector(1, (long)-1),  bool  log=true);

         bool purge();

         bool recover( bool  log=true);

         long length();

         std::vector<long> indices();

         bool sort(const  string& criteria=string("Flux"),  bool  log=true);

         bool isphysical( const std::vector<long>& which=initialize_vector(1, (long)-1));

         std::vector<double>sample( const variant & direction= variant(),  const variant & pixellatsize= variant(),  const variant & pixellongsize= variant(),  const variant & frequency= variant());

         bool rename(const  string& filename,  bool  log=true);

         bool simulate( long  howmany=1,  bool  log=true);

         bool addcomponent( const variant & flux= variant(), const  string& fluxunit=string("Jy"), const  string& polarization=string("Stokes"),  const variant & dir= variant(), const  string& shape=string("point"),  const variant & majoraxis= variant(),  const variant & minoraxis= variant(),  const variant & positionangle= variant(),  const variant & freq= variant(), const  string& spectrumtype=string("constant"), 
			  double index=1.0,  const std::vector<double>& optionalparms=initialize_vector(1, (double)0.0), const  string& label=string(""));

         bool close( bool  log=true);

         bool edit( long  which,  bool  log=true);

         bool done();

         bool select( const std::vector<long>& which);

         bool deselect( const std::vector<long>& which);

         std::vector<long> selected();

         string getlabel( long  which);

         bool setlabel( long  which, const  string& value,  bool  log=true);

         std::vector<double>getfluxvalue( long  which);

         string getfluxunit( long  which);

         string getfluxpol( long  which);

         std::vector<double>getfluxerror( long  which);

         bool setflux( long  which,  const variant & value= variant(), const  string& unit=string("Jy"), const  string& polarization=string("Stokes"),  const variant & error= variant(),  bool  log=true);

         bool convertfluxunit( long  which, const  string& unit=string("Jy"));

         bool convertfluxpol( long  which, const  string& polarization=string("Stokes"));

         record* getrefdir( long  which);

         string getrefdirra( long  which, const  string& unit=string("deg"),  long  precision=6);

         string getrefdirdec( long  which, const  string& unit=string("deg"),  long  precision=6);

         string getrefdirframe( long  which);

         bool setrefdir( long  which=1,  const variant & ra= variant(),  const variant & dec= variant(),  bool  log=true);

         bool setrefdirframe( long  which, const  string& frame,  bool  log=true);

         bool convertrefdir( long  which, const  string& frame);

         string shapetype( long  which);

         record* getshape( long  which);

         bool setshape( long  which, const  string& type=string("Point"),  const variant & majoraxis= variant(),  const variant & minoraxis= variant(),  const variant & positionangle= variant(),  const variant & majoraxiserror= variant(),  const variant & minoraxiserror= variant(),  const variant & positionangleerror= variant(),  const std::vector<double>& optionalparms=initialize_vector(1, (double)0.0),  bool  log=true);

         bool convertshape( long  which, const  string& majoraxis=string("arcmin"), const  string& minoraxis=string("arcmin"), const  string& positionangle=string("deg"));

         string spectrumtype( long  which);

         record* getspectrum( long  which);

         bool setstokesspectrum( long  which, const  string& type=string("spectral index"),  const std::vector<double>& index=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularfreqs=initialize_vector(1,(double)1.0e11),  const std::vector<double>& tabulari=initialize_vector(1,(double)1.0),  const std::vector<double>& tabularq=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularu=initialize_vector(1,(double)0.0),  const std::vector<double>& tabularv=initialize_vector(1,(double)0.0),  const variant & reffreq= variant(), const  string& frame=string("LSRK"));

         bool setspectrum( long  which, const  string& type=string("spectral index"), 
			  double index=0.0,  const std::vector<double>& tabularfreqs=initialize_vector(1,(double)1.0e11),  const std::vector<double>& tabularflux=initialize_vector(1,(double)1.0), const  string& tabularframe=string("LSRK"));

         record* getfreq( long  which);

         double getfreqvalue( long  which);

         string getfrequnit( long  which);

         string getfreqframe( long  which);

         bool setfreq( long  which, 
			  double value, const  string& unit=string("GHz"),  bool  log=true);

         bool setfreqframe( long  which, const  string& frame=string("LSRK"),  bool  log=true);

         bool convertfrequnit( long  which, const  string& unit=string("GHz"));

         record* getcomponent( long  which,  bool  iknow=false);

         bool add( const record& thecomponent,  bool  iknow=true);

         bool replace( long  which,  const record& list,  const std::vector<long>& whichones=initialize_vector(1, (long)-1));

         bool summarize( long  which=-1);

         bool iscomponentlist( const variant & tool);


      private :
#include <tools/components/componentlist_private.h>
   }; 

 
}
#endif
